#ifndef CONJ_EST_HH
#define CONJ_EST_HH

#include <vector>
#include <iostream>
#include "Estudiant.hh"

using namespace std;

class Cjt_estudiants {

private:

  // Representa un conjunt d'estudiants ordenat per DNI.  
  
  vector<Estudiant> vest; // vector amb els estudiants del conjunt 
  int nest; // nombre d'estudiants del conjunt
  static const int MAX_NEST = 2500005; // nombre maxim d'estudiants del conjunt
  int imax; // posici de l'estudiant amb nota mxima

  /*
     Invariant de la representaci: 
     - vest[0..nest-1] est ordenat creixentment pel dni dels estudiants i no cont estudiants repetits
     - 0 <= nest <= vest.size() = MAX_NEST
     - imax s la posici de l'estudiant amb nota mxima a vest[0..nest-1];
       cas que hi hagi ms d'un estudiant amb nota mxima, imax assenyala el de dni menor 
       si no hi ha cap estudiant amb nota imax val -1
  */

  void ordenar();
  /* Pre: cert */
  /* Post: el elements del parmetre implcit passen a estar ordenats
     creixentment per DNI. */
     
  static int cerca_dicot(const vector<Estudiant> &vest, int left, int right, int x);
  /* Pre: els elements de vest estan ordenats creixentment per DNI,
     0<=left, right<vest.size(), right>=left+1 */
  /* Post: si x hi s vest[left..right], el resultat s la posici que
     hi ocupa, si no, s la posici que hauria d'ocupar */
 
  void recalcular_posicio_imax();
  /* Pre: el campo imax del parmetro implcito no necesariamente tiene
     el valor que le corresponde */
  /* Post: el camp imax cont la posici de l'element del parmetre implcit
     amb la nota ms gran; si no hi ha cap estudiant amb nota, val -1 */
  
public:

  //Constructores

  Cjt_estudiants();
  /* Pre: cert */
  /* Post: crea un conjunt d'estudiants buit */

  //Modificadores

  void afegir_estudiant(const Estudiant &est, bool& b);
  /* Pre: el parmetre implcit no est ple */
  /* Post: b = indica si el p.i. original cont un estudiant amb el dni d'est;
     si b = fals, s'ha afegit l'estudiant est al parmetre implcit */

  void modificar_estudiant(const Estudiant &est, bool& b);
  /* Pre: cert */
  /* Post: b indica si el parmetre implcit original tenia un estudiant 
     amb el dni d'est; si b, aquest ha quedat sustituit per est */
  
  void unir_conjunts(const Cjt_estudiants & c);
  
  void actualitzar_conjunt();
  
  //Consultores
	
  int mida() const; 
  /* Pre: cert */
  /* Post: El resultat s el nombre d'estudiants del parmetre implcit */

  static int mida_maxima();
  /* Pre: cert */
  /* Post: el resultat s el nombre maxim d'estudiants que pot arribar
     a tenir el parmetre implcit */
  
  Estudiant estudiant_nota_max() const;
  /* Pre: el parmetre implcit cont almenys un estudiant amb nota */
  /* Post: el resultat s l'estudiant del parmetre implcit amb nota
     mxima; si en t ms d'un, s el de DNI ms petit */ 
 
  // Lectura i escriptura
	
  void llegir();
  /* Pre: el canal estndar d'entrada cont un enter positiu n seguit de 
     n descripcions d'estudiants, i.e. parells (dni, nota), ordenats 
      creixentment per dni */
  /* Post: el parmetre implcit cont el conjunt d'estudiants llegits
     del canal estndar d'entrada */

  void escriure() const;
  /* Pre: cert */
  /* Post: s'han escrit pel canal estndar de sortida 
     els estudiants del conjunt que cont el parmetre 
     implcit en ordre ascendent per DNI */

};

#endif

